#!@WHICHPERL@ -w

# Author: Timothy Bailey

# This script copies the list of supported databases to
# the client. It is intended to support clients using 
# XMLHttpRequest to update a selection list of supported databases
# on a web page.

use lib qw(@PERLLIBDIR@);
require "Utils.pm";                     # must come before "whines"
use CGI qw/:standard/;
use CGI::Carp qw(fatalsToBrowser set_message);

# Set standard message for error reporting.
set_message('Please report this message to <a href="mailto:@contact@">@contact@</mailto>.');

$csv_file = param('db_names');
$short_only = param('short_only');	# only include databases with "short_seqs"
					# field set to "yes"
$doc = param('doc');			# print documentation only if true
$is_motif_db = param('is_motif_db');

my $dir = "@MEME_DIR@";	# Installation directory
my $cgi = new CGI;

# Open list of supported databases.
my $result = open(DB_NAMES, "<$dir/etc/$csv_file"); 

# Print desired contents (names or documentation)
if (not $result) {
  print $cgi->header(-status=>'500 Internal Server Error');
  die "Unable to open the list of supported databases.\n";
} else {
  if ($doc) {
    doc_print();
  } else {
    list_print();
  }
  close(DB_NAMES);
}

#
# Output the list of databases
#
sub list_print {
  print "Content-type: text/plain\n\n";
  while (<DB_NAMES>) {
    next if (/^#/ || /^\s*$/);		# skip comments and whitespace
    chomp;
    my @fields = split /,/;
    # remove invisible whitespace from name and yes/no fields
    my $i;
    for ($i=0; $i<@fields; $i++) {
      next if $i >= 4 && $i < 8 || $i > 8 && $i % 2;
      $fields[$i] =~ s/\s+//g;
    }
    #next if ($short_only && $fields[3] eq 'no');
    $fields[2] = 'no' if ($short_only && $fields[3] eq 'no');	# turn off long DNA if need be
    # create a list of types and add to name if not a header
    if ($fields[0] =~ /\S/ && $fields[4] =~ /\S/) {
      my $db_types;
      if ($is_motif_db) {
        $db_types = "";
      } elsif ($fields[1] eq 'yes' && $fields[2] eq 'yes') {
        $db_types = "(peptide and nucleotide)"; 
      } elsif ($fields[1] eq 'yes') {
        $db_types = "(peptide only)"; 
      } elsif ($fields[2] eq 'yes') {
        $db_types = "(nucleotide only)"; 
      } else {
        $db_types = "(unknown database type)"; 
      }
      $fields[4] .= " $db_types";
    }
    print join(",", @fields) . "\n";
  }
} # list_print 

#
# Output the documentation on supported databases.
# 
sub doc_print {
  # read in the file
  my @lines = <DB_NAMES>; 
 
  # get the number of separator lines
  my @categories = ();
  my @descriptions = ();
  for $_ (@lines) {
    next if (/^#/ || !/\S/);		# skip comments and blank lines
    chomp;
    my @fields = split(/,/);
    #next if ($short_only && $fields[3] eq 'no');	# skip categories with only long seqs
    $fields[2] = 'no' if ($short_only && $fields[3] eq 'no');	# turn off long DNA if need be
    if ($fields[0] !~ /\S/ && $fields[4] =~ /\S/) { # check for category
      $fields[4] =~ s/^-+//g; 
      $fields[4] =~ s/-+$//g;
      push(@categories, $fields[4]);    #db menu name
      push(@descriptions, $fields[5]);  #db long name
    }
  } 
  # set globals
  if ($is_motif_db) {
    $db_type1 = "Motif";
    $db_type2 = "motif";
  } else {
    $db_type1 = "Sequence";
    $db_type2 = "sequence";
  }
  my $tab = " ";
  my $form = "";
  my $tabx3 = $tab x 3;
  my $css = 
      $tabx3."h4 {\n".
      $tabx3.$tab."margin-bottom: 0;\n".
      $tabx3."}\n".
      $tabx3."h5 {\n".
      $tabx3.$tab."font-size: 14px;\n".
      $tabx3.$tab."font-family: Arial;\n".
      $tabx3.$tab."font-weight: bold;\n".
      $tabx3."}\n".
      $tabx3.".db {\n".
      $tabx3.$tab."margin-left: 1.5em;\n".
      $tabx3.$tab."margin-top: 0px;\n".
      $tabx3.$tab."margin-bottom: 0px;\n".
      $tabx3."}\n".
      $tabx3."#main ul {\n".
      $tabx3.$tab."margin-top: 0;\n".
      $tabx3.$tab."margin-bottom: 0;\n".
      $tabx3."}\n".
      $tabx3."#main li {\n".
      $tabx3.$tab."font-size: 12px;\n".
      $tabx3."}\n";

  $form .= make_form_header("$db_type1 Databases", "Documentation", $css, 0, $tab);
  $form .= doc_start_body(\@categories, \@descriptions, 1, $tab);
  $form .= doc_add_dbs(\@lines, 8, $tab);
  $form .= doc_finish_body(1, $tab);
  $form .= "</html>\n";

  print "Content-type: text/html\n\n$form";
} # doc_print

sub doc_start_body {
  my ($categories_p, $descriptions_p, $indent_lvl, $tab) = @_;

  $indent_lvl = 0 unless $indent_lvl;
  $tab = "\t" unless $tab;

  my $indent = $tab x $indent_lvl;

  # create list of categories
  my ($i, $cat_descr);
  my $ncats = @{$categories_p};
  my $desc_indent = $indent . ($tab x 6);
  for ($i=1; $i<=$ncats; $i++) {
    my $cat = $categories_p->[$i-1];
    my $descr = $descriptions_p->[$i-1];
    $cat_descr .= $desc_indent."<li><h3><a href=#cat$i>$cat</a></h3>\n";
    $cat_descr .= $desc_indent.$descr."\n";
  }
  my $groups_desc = "$ncats categories";
  if ($ncats == 1) {
    $groups_desc = "1 category";
  }

  my $content = 
      $indent."<body class=\"body\">\n".
      $indent.$tab."<table class=\"maintable\">\n".
      $indent.$tab.$tab."<tr>\n".
      $indent.$tab.$tab.$tab."<td class=\"maintablewidth\">\n".
      $indent.$tab.$tab.$tab.$tab."<div id=\"main\">\n".
      $indent.$tab.$tab.$tab.$tab.$tab."<script src=\"../doc/meme-suite-logo.js\" type=\"text/javascript\"></script>\n".
      $indent.$tab.$tab.$tab.$tab.$tab."<noscript>\n".
      $indent.$tab.$tab.$tab.$tab.$tab.$tab."<h1>MEME Suite</h1>\n".
      $indent.$tab.$tab.$tab.$tab.$tab.$tab."The MEME Suite web application requires the use of JavaScript<br />\n".
      $indent.$tab.$tab.$tab.$tab.$tab.$tab."Javascript doesn't seem to be available on your browser.\n".
      $indent.$tab.$tab.$tab.$tab.$tab."</noscript>\n".
      $indent.$tab.$tab.$tab.$tab.$tab."<hr />\n".
      $indent.$tab.$tab.$tab.$tab.$tab."<h1 align=\"center\">$db_type1 databases available for search</h1>\n".
      $indent.$tab.$tab.$tab.$tab.$tab."<hr />\n".
      $indent.$tab.$tab.$tab.$tab.$tab."<h3>The $db_type2 databases that can be searched are grouped into $groups_desc:</h3>\n".
      $indent.$tab.$tab.$tab.$tab.$tab."<ul>\n".
      $cat_descr.
      $indent.$tab.$tab.$tab.$tab.$tab."</ul>\n";
} # doc_start_body

sub doc_add_dbs {
  my ($lines, $indent_lvl, $tab) = @_;
  $indent_lvl = 0 unless $indent_lvl;
  $tab = "\t" unless $tab;
  my $indent = $tab x $indent_lvl;

  my $i = 1;
  my $content;

  foreach $_ (@$lines) {
    next if (/^#/ || !/\S/);		# skip comments and blank lines
    chomp;
    my @fields = split(/,/);
    $fields[2] = 'no' if ($short_only && $fields[3] eq 'no');	# remove DNA if need be
    my @extra_dbs = ();
    my $ind;
    for ($ind = 9; $ind < @fields; $ind += 2) {
      push(@extra_dbs, @fields[$ind]);
    }
    if ($fields[0] !~ /\S/) {
      $fields[4] =~ s/^-+//g; $fields[4] =~ s/-+$//g;
      $content .= 
          $indent."<hr />\n".
          $indent."<a name=\"cat$i\"></a>\n".
          $indent."<h3>$fields[4]</h3>\n".
          $indent."<hr />\n";
      $i++;
    } else {
      my $db_types;
      if ($is_motif_db) {
        $db_types = "";
      } else {
        if ($fields[1] eq 'yes' && $fields[2] eq 'yes') {
          $db_types = "peptide and nucleotide"; 
        } elsif ($fields[1] eq 'yes') {
          $db_types = "peptide only"; 
        } elsif ($fields[2] eq 'yes') {
          $db_types = "nucleotide only"; 
        } else {
          $db_types = "unknown database type"; 
        }
      }
      $content .= 
          $indent.$tab."<h4>$fields[4]</h4>\n".
          $indent.$tab."<p class=\"db\">$fields[5]</p>\n";
      $content .= $indent.$tab."<p class=\"db\" >($db_types)</p>\n" unless ($is_motif_db); 
      if (@extra_dbs > 0) {
        $content .=
            $indent.$tab."<h5 class=\"db\">Comparative database(s):</h5>\n".  
            $indent.$tab."<ul>\n";
        foreach $db (@extra_dbs) {
          $content .=
              $indent.$tab.$tab."<li>$db</li>\n";
        }
        $content .=
            $indent.$tab."</ul>\n";
      }
    }
  }
  $content .= 
      $indent."<hr />\n";

  return($content);
} # doc_add_dbs

sub doc_finish_body {
  my ($indent_lvl, $tab) = @_;
  $indent_lvl = 0 unless $indent_lvl;
  $tab = "\t" unless $tab;
  my $indent = $tab x $indent_lvl;

  my $content .= 
      $indent.$tab.$tab.$tab.$tab.$tab."<script src=\"../template-footer.js\" type=\"text/javascript\"></script>\n".
      $indent.$tab.$tab.$tab.$tab."</div>\n".
      $indent.$tab.$tab.$tab."</td>\n".
      $indent.$tab.$tab."</tr>\n".
      $indent.$tab."</table>\n".
      $indent."</body>\n";

  return($content);
} # doc_finish_body
